#include <array>

#include "App.hpp"
#include "Math.hpp"

#include "raylib.h"
#include "raymath.h"

App::App(int screenWidth, int screenHeight) : screenWidth(screenWidth), screenHeight(screenHeight), trees({Tree(screenWidth), Tree(screenWidth)})
{
}

void App::init()
{
  for (auto &&tree : trees)
    tree.newTree();

  center = {(float)screenWidth / 2, (float)screenHeight / 2};
  float rotation = 0.0f;
  destA = CalculateRect(center, rotation, screenWidth, screenWidth);
  destB = {destA.x, destA.y, (float)screenWidth, (float)screenWidth};
}

void App::update()
{
  if (IsMouseButtonDown(MOUSE_BUTTON_LEFT))
  {
    int mouseX = GetMouseX();
    center.x = mouseX;
    center.y = GetMouseY();
    float l = (float)mouseX / screenWidth;
    rotation = Lerp(45.0f, -45.0f, l);
    destA = CalculateRect(center, rotation, screenWidth, screenWidth);
  }

  if (IsMouseButtonReleased(MOUSE_BUTTON_LEFT))
  {
    trees[1 - pos].newTree();
    pos = 1 - pos;
    rotation = 0.0f;
    center.x = screenWidth / 2;
    center.y = screenHeight / 2;
    destA = CalculateRect(center, rotation, screenWidth, screenWidth);
  }
}

void App::draw()
{
  ClearBackground(RED);
  trees[pos].draw(destB, 0.0f);
  trees[1 - pos].draw(destA, rotation);
}

void App::deinit()
{
  for (size_t i = 0; i < trees.size(); i++)
  {
    trees[i].deinit();
  }
}